from rest_framework import serializers


class RecycleSerializer(serializers.Serializer):
    model_name          = serializers.SerializerMethodField(read_only=True)
    name                = serializers.SerializerMethodField(read_only=True)
    pk                  = serializers.SerializerMethodField(read_only=True)
    deleted_by          = serializers.SerializerMethodField(read_only=True)
    deleted_at          = serializers.SerializerMethodField(read_only=True)

    def get_model_name(self, obj):
        try:
            return obj._meta.verbose_name.replace(" ", "")
        except:
            return None
    def get_name(self, obj):
        try:
            return obj.name
        except:
            return None        
    def get_pk(self, obj):
        try:
            return obj.id
        except:
            return None

    def get_deleted_by(self, obj):
        try:
            return str(obj.deleted_by)
        except:
            return None

    def get_deleted_at(self, obj):
        try:
            return obj.deleted_at
        except:
            return None


from django.contrib.contenttypes.models import ContentType
from .serializers import RecycleSerializer




def get_deleted_items(models):
    qs_li = []
    for model in models:
        qs = model.objects.filter(deleted=True).order_by('deleted_at')
        qs_li.append(qs)
    serializer = RecycleSerializer(qs_li, many=True)
    return serializer.data



def restore(model, id):
    ct = ContentType.objects.get(model=model)
    model = ct.model_class()
    model.recycler.filter(pk=id).update(deleted=False)



def permanent_delete(model, id):
    ct = ContentType.objects.get(model=model)
    model = ct.model_class()
    obj = model.recycler.get(pk=id)
    obj.permdelete()




from django.db import models
from django.db.models import Manager
from .utils import get_current_request
from django.utils import timezone
from django.contrib.auth import get_user_model
User = get_user_model()

class SoftDeleteManager(models.QuerySet):
    def get(self, **kwargs):
        return super().get(**kwargs,deleted=False)
    def all(self):
        return super().all().filter(deleted=False)

    def get_query_set(self, *args ,**kwargs):
        return super().get_query_set().filter(deleted=False)






class SoftDeleteMixin(models.Model):
    deleted = models.BooleanField(default=False)
    deleted_at = models.DateTimeField(blank=True, null=True)
    deleted_by = models.ForeignKey(User, related_name='%(class)s_requests_created', on_delete=models.CASCADE, blank=True, null=True)
    class Meta:
        abstract=True

    objects = SoftDeleteManager.as_manager()
    recycler = Manager()

    def delete(self):
        request = get_current_request()
        self.deleted_at = timezone.now()
        try:
            self.deleted_by = request.user
        except:
            pass
        self.deleted=True
        self.save()

    def permdelete(self):
        return super().delete()



from threading import current_thread
from django.utils.deprecation import MiddlewareMixin
from django.utils.encoding import force_text
from rest_framework import status
from rest_framework.exceptions import APIException

_requests = {}

def get_current_request():
    t = current_thread()
    if t not in _requests:
        return None
    return _requests[t]


class RequestMiddleware(MiddlewareMixin):
    def process_request(self, request):
        _requests[current_thread()] = request


class CustomValidation(APIException):
    status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
    default_detail = 'A server error occurred.'

    def __init__(self, detail, field, status_code):
        if status_code is not None:
            self.status_code = status_code
        if detail is not None:
            self.detail = {field: force_text(detail)}
        else: 
            self.detail = {'detail': force_text(self.default_detail)}